/*
 * Copyright 2014-2015 ISP RAS (http://www.ispras.ru)
 * 
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except
 * in compliance with the License. You may obtain a copy of the License at
 * 
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software distributed under the License
 * is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express
 * or implied. See the License for the specific language governing permissions and limitations under
 * the License.
 */

package ru.ispras.testbase.storage.internal;

import java.util.Map;
import java.util.HashMap;
import java.util.Iterator;

import java.lang.Iterable;

import ru.ispras.testbase.TestData;
import ru.ispras.testbase.TestDataProvider;

import ru.ispras.fortress.expression.Node;
import ru.ispras.fortress.expression.NodeValue;
import ru.ispras.fortress.expression.NodeVariable;

import ru.ispras.fortress.data.Data;

class RawDataProvider extends TestDataProvider {
  final Iterator<Map<String, String>> source;
  final Map<String, Node> image;

  RawDataProvider(Iterable<Map<String, String>> dataset, Map<String, Node> bindings) {
    this.source = dataset.iterator();
    this.image = bindings;
  }

  public boolean hasNext() {
    return source.hasNext();
  }

  public TestData next() {
    final Map<String, String> dataMap = source.next();
    final Map<String, Node> bindings = new HashMap<String, Node>(image);

    for (String key : bindings.keySet()) {
      final Node node = bindings.get(key);

      if (node.getKind() != Node.Kind.VARIABLE) {
        continue;
      }

      final NodeVariable var = (NodeVariable) node;
      final String stored = dataMap.get(var.getName());
      final Data data = var.getDataType().valueOf(stored, 10);

      bindings.put(key, new NodeValue(data));
    }

    return new TestData(bindings);
  }
}
